/// HOW TO USE THIS SCRIPT:

/// 1. Move the following call to ShutdownPwi4() to someplace in the main() function in ShutdownObs.js
ShutdownPwi4();

/// 2. Move all of the following code to the end of the ShutdownObs.js file

/// 3. Edit the sequence of shutdown steps below (e.g. TurnOffFans(), CloseMirrorCover())
///    to enable or disable features as needed.

/// BEGIN code for shutting down PlaneWave hardware
function ShutdownPwi4()
{
    // BEGIN CONFIGURATION

    var PWI4_PORT = 8220;  // CHANGE THIS to the port that PWI4 is configured to use for its server

    // END CONFIGURATION

    // BEGIN PWI Shutdown Script.
    // Comment or uncomment the following as needed.
    // Any line beginning with 2 forward slashes "//" will be SKIPPED.

    LaunchPwi4();     // Make sure PWI4 is still running
    ConnectToMount(); // Need to be connected in order to turn off motors (and fans for PW1000)

    TurnOffMountMotors();

    ConnectToFocuser();      // Connect to Series 5 so that we can turn off fans/focuser/rotator
    TurnOffFans();           // PW1000 or Series 5
    TurnOffFocuserMotor();   // PW1000 or Series 5
    TurnOffRotatorMotor();   // PW1000 or Series 5

    DisconnectMount();
    DisconnectFocuser();

    //CloseMirrorCover();      // Only if an automated mirror cover is installed
    //ShutdownMirrorCover();   // Close the PlaneWave Shutter Control software


    // END PWI Shutdown Script


    // BEGIN high-level functions.

    function LaunchPwi4()
    {
        var pwi4Path = "C:\\Program Files (x86)\\PlaneWave Instruments\\PlaneWave Interface 4\\PWI4.exe";

        RunProg(pwi4Path);
        Sleep(5000); // Wait a bit for the program to finish launching
    }

    function CloseMirrorCover()
    {
        var pwshutterPath = "C:\\Program Files (x86)\\PlaneWave Instruments\\PlaneWave Shutter Control\\pwshutter.exe";

        WriteLog("Connecting to mirror cover");
        try {
            var exitCode = RunProg(pwshutterPath + " connect", true);
            if (exitCode != 0) {
                throw new Error(0x80040001, "Failed to connect to PWShutter");
            }
        }
        catch (ex) {
            WriteLog("ex1 " + ex.message);
            throw ex;
        }
        Sleep(5000);

        WriteLog("Closing mirror cover");
        try {
            var exitCode = RunProg(pwshutterPath + " close", true);
            if (exitCode != 0) {
                throw new Error(0x80040001, "Failed to close mirror covers using PWShutter");
            }
        }
        catch (ex) {
            WriteLog("ex2 " + ex.message);
            throw ex;
        }
    }

    function ShutdownMirrorCover()
    {
        WriteLog("Shutting down mirror cover");
        try {
            var exitCode = RunProg("taskkill /im pwshutter.exe", true);
            if (exitCode != 0) {
                throw new Error(0x80040001, "Failed to shut down PWShutter");
            }
        }
        catch (ex) {
            WriteLog("ex1 " + ex.message);
            throw ex;
        }
    }

    function ConnectToMount()
    {
        // Assuming PWI4 is already running, send the command
        // to connect PWI4 to the mount

        WriteLog("Connecting to mount...");
        PwiRequest("/mount/connect");

        WriteLog("Checking mount connection...");
        var response = PwiRequest("/status");

        if (ResponseHasValue(response, "mount.is_connected", "true")) {
            WriteLog("CONNECTED!!");
        }
        else {
            throw new Error(0x80040001, "===== Failed to connect to mount");
        }
    }

    function ConnectToFocuser()
    {
        // Assuming PWI4 is already running, send the command
        // to connect PWI4 to the focuser. For devices where the
        // focuser and rotator share a single connection (such
        // as most Series 5 hardware), this will also result in the
        // rotator being connected

        WriteLog("Connecting to focuser...");
        PwiRequest("/focuser/connect");

        // Wait a moment for connection
        Sleep(3000);

        WriteLog("Checking focuser connection...");
        var response = PwiRequest("/status");

        if (ResponseHasValue(response, "focuser.is_connected", "true")) {
            WriteLog("CONNECTED!!");
        }
        else {
            throw new Error(0x80040001, "===== Failed to connect to focuser");
        }
    }

    function TurnOffFans()
    {
        WriteLog("Turning off fans...");
        PwiRequest("/fans/off");
    }

    function TurnOffMountMotors()
    {
        WriteLog("Disabling azimuth motor...");
        PwiRequest("/mount/disable?axis=0");

        WriteLog("Disabling altitude motor...");
        PwiRequest("/mount/disable?axis=1");
    }

    function TurnOffFocuserMotor()
    {
        // TODO: Disable focuser1 and focuser2 explicitly. Currently enables whichever one M3 is pointing at
        WriteLog("Disabling focuser motor...");
        PwiRequest("/focuser/disable");
    }

    function TurnOffRotatorMotor()
    {
        // TODO: Disable rotator1 and rotator2 explicitly. Currently enables whichever one M3 is pointing at
        WriteLog("Disabling rotator motor...");
        PwiRequest("/rotator/disable");
    }

    function DisconnectMount() {
        WriteLog("Disconnecting PWI4 from mount...");
        PwiRequest("/mount/disconnect");
    }

    function DisconnectFocuser() {
        WriteLog("Disconnecting PWI4 from focuser controller...");
        PwiRequest("/focuser/disconnect");
    }

    // END HIGH LEVEL FUNCTIONS

    // BEGIN UTILITY FUNCTIONS used for talking to PWI

    function PwiRequest(command) {
        var http = new ActiveXObject("MSXML2.ServerXMLHTTP");
        var url = "http://localhost:" + PWI4_PORT + command;

        try {
            http.open("GET", url, false);
            http.setRequestHeader("Cache-Control", "max-age=0"); // Avoid getting a previous cached response back
            http.send();
        }
        catch (ex) {
            throw new Error(0x80040001, "PWI request failed. Is PWI running? Is the server enabled? Is the correct port being used?");
        }

        var responseText = http.responseText;
        return http.responseText;
    }

    function ResponseHasValue(response, keyword, desiredValue)
    {
        // Search keyword=value text in response for:
        var regex = "^"                    // The beginning of a line
                  + keyword                // Followed by the keyword
                  + "="                    // Followed by an "equals" sign
                  + desiredValue           // Followed by the desired value
                  + "$"                    // Followed by the end of a line
                  ;

        var multilineFlag = "m"; // Allow "^" and "$" to work in a multiline string

        return new RegExp(regex, multilineFlag).test(response);
    }

    function WriteLog(text) {
        if (typeof(WScript) === "object") {
            // Running via cscript
            WScript.Echo(text);
        }
        else if (typeof(Console) === "object" && typeof(Console.PrintLine) !== "undefined") {
            // Running via ACP
            Console.PrintLine(text);
        }
    }

    function Sleep(msec) {
        if (typeof(WScript) === "object") {
            // Running via cscript
            WScript.Sleep(msec);
        }
        else if (typeof(Util) === "object" && typeof(Util.WaitForMilliseconds) !== "undefined") {
            // Running via ACP
            Util.WaitForMilliseconds(msec);
        }
    }

    function RunProg(command, waitForReturn) {
        var wshRunning = 0;
        var wshFinished = 1;
        var wshFailed = 2;

        var wshShell = new ActiveXObject("WScript.Shell");

        WriteLog("Running " + command);
        var process = wshShell.Exec(command);

        if (! waitForReturn) {
            return; // Don't wait for program to exit
        }

        while (process.Status == wshRunning) {
            Sleep(100);
        }

        WriteLog("Exited with code " + process.ExitCode);

        if (process.Status == wshFailed) {
            WriteLog("FAILED to run");
        }

        return process.ExitCode;
    }

    // END UTILITY FUNCTIONS

}
/// END code for initializing PlaneWave hardware
